﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace FourDotsTimePositionControlDemo
{
    public partial class FourDotsTimePositionControl : UserControl
    {                                
        public TimeSpan MaxTime { get; set; }        
        public TimeSpan MinTime { get; set; }

        private int _Hours = 0;
        private int _Minutes = 0;
        private int _Seconds = 0;
        private int _MSeconds = 0;

        private bool DontFillTextBoxes = false;

        public int Hours
        {
            get
            {
                return _Hours;
            }
            set
            {
                TimeSpan ts = new TimeSpan(0,value, _Minutes, _Seconds, _MSeconds);

                if (ts.CompareTo(MaxTime) <= 0 && ts.CompareTo(MinTime) >= 0)
                {                    
                    _Hours = value;
                    FillTimeTextBoxes();
                }

            }
        }

        public int Minutes
        {
            get
            {
                return _Minutes;
            }
            set
            {
                TimeSpan ts = new TimeSpan(0, _Hours,value, Seconds, _MSeconds);

                if (ts.CompareTo(MaxTime) <= 0 && ts.CompareTo(MinTime) >= 0)
                {
                    _Minutes = value;
                    FillTimeTextBoxes();
                }

            }
        }

        public int Seconds
        {
            get
            {
                return _Seconds;
            }
            set
            {
                TimeSpan ts = new TimeSpan(0, _Hours, _Minutes, value, _MSeconds);

                if (ts.CompareTo(MaxTime) <= 0 && ts.CompareTo(MinTime) >= 0)
                {
                    _Seconds = value;
                    FillTimeTextBoxes();
                }

            }
        }

        public int MSeconds
        {
            get
            {
                return _MSeconds;
            }
            set
            {
                TimeSpan ts = new TimeSpan(0, _Hours, _Minutes, _Seconds, value);

                if (ts.CompareTo(MaxTime) <= 0 && ts.CompareTo(MinTime) >= 0)
                {
                    _MSeconds = value;
                    FillTimeTextBoxes();
                }

            }
        }

        public TimeSpan TimeSpan
        {
            get
            {
                return new TimeSpan(0, Hours, Minutes, Seconds, MSeconds);
            }
            set
            {
                Hours = value.Hours;
                Minutes = value.Minutes;
                Seconds = value.Seconds;
                MSeconds = value.Milliseconds;

                FillTimeTextBoxes();
            }
        }

        private void FillTimeTextBoxes()
        {
            if (!DontFillTextBoxes)
            {
                txtH.Text = Hours.ToString("D2");
                txtM.Text = Minutes.ToString("D2");
                txtS.Text = Seconds.ToString("D2");
                txtMS.Text = MSeconds.ToString("D3");
            }
        }

        public FourDotsTimePositionControl()
        {
            InitializeComponent();
            MinTime = new TimeSpan(0);
            //MaxTime = new TimeSpan(0);

            MaxTime = new TimeSpan(10, 10, 12);

            btnHDown.MouseDown += btnTime_MouseDown;
            btnHDown.MouseUp += btnTime_MouseUp;
            btnHUp.MouseDown += btnTime_MouseDown;
            btnHUp.MouseUp += btnTime_MouseUp;

            btnMDown.MouseDown += btnTime_MouseDown;
            btnMDown.MouseUp += btnTime_MouseUp;
            btnMUp.MouseDown += btnTime_MouseDown;
            btnMUp.MouseUp += btnTime_MouseUp;

            btnSDown.MouseDown += btnTime_MouseDown;
            btnSDown.MouseUp += btnTime_MouseUp;
            btnSUp.MouseDown += btnTime_MouseDown;
            btnSUp.MouseUp += btnTime_MouseUp;

            btnMSDown.MouseDown += btnTime_MouseDown;
            btnMSDown.MouseUp += btnTime_MouseUp;
            btnMSUp.MouseDown += btnTime_MouseDown;
            btnMSUp.MouseUp += btnTime_MouseUp;

            txtH.MouseWheel += txtH_MouseWheel;
            txtM.MouseWheel += txtM_MouseWheel;
            txtS.MouseWheel += txtS_MouseWheel;
            txtMS.MouseWheel += txtMS_MouseWheel;
        }

        #region Mouse Wheel Event Handlers

        void txtMS_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                btnMSUpClick();
            }
            else if (e.Delta < 0)
            {
                btnMSDownClick();
            }
        }

        void txtS_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                btnSUpClick();
            }
            else if (e.Delta < 0)
            {
                btnSDownClick();
            }
            
        }

        void txtM_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                btnMUpClick();
            }
            else if (e.Delta < 0)
            {
                btnMDownClick();
            }
        }

        void txtH_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                btnHUpClick();
            }
            else if (e.Delta < 0)
            {
                btnHDownClick();
            }
        }

        #endregion

        void btnTime_MouseDown(object sender, MouseEventArgs e)
        {
            Button btn = (Button)sender;

            btn.Tag = "MouseDown";

            if (btn == btnHUp)
            {
                btnHUpClick();
            }
            else if (btn == btnHDown)
            {
                btnHDownClick();
            }
            else if (btn == btnMUp)
            {
                btnMUpClick();
            }
            if (btn == btnMDown)
            {
                btnMDownClick();
            }
            if (btn == btnSUp)
            {
                btnSUpClick();
            }
            if (btn == btnSDown)
            {
                btnSDownClick();
            }
            if (btn == btnMSUp)
            {
                btnMSUpClick();
            }
            if (btn == btnMSDown)
            {
                btnMSDownClick();
            }            

            while (btn.Tag.ToString() == "MouseDown")
            {
                WaitNMSeconds(700);

                if (btn.Tag.ToString() == "MouseDown")
                {
                    if (btn == btnHUp)
                    {
                        btnHUpClick();
                    }
                    else if (btn == btnHDown)
                    {
                        btnHDownClick();
                    }
                    else if (btn == btnMUp)
                    {
                        btnMUpClick();
                    }
                    if (btn == btnMDown)
                    {
                        btnMDownClick();
                    }
                    if (btn == btnSUp)
                    {
                        btnSUpClick();
                    }
                    if (btn == btnSDown)
                    {
                        btnSDownClick();
                    }
                    if (btn == btnMSUp)
                    {
                        btnMSUpClick();
                    }
                    if (btn == btnMSDown)
                    {
                        btnMSDownClick();
                    }
                }
            }
        }

        void btnTime_MouseUp(object sender, MouseEventArgs e)
        {
            Button btn = (Button)sender;
            btn.Tag = "MouseUp";
        }        

        private void FourDotsTimePositionControl_Load(object sender, EventArgs e)
        {

        }

        #region Up Down Click Events

        private void btnHUpClick()
        {
            Hours++;
        }

        private void btnHDownClick()
        {
            if (Hours > 0)
            {
                Hours--;
            }
        }

        private void btnMUpClick()
        {
            if (Minutes < 59)
            {
                Minutes++;
            }
            else
            {
                int lasth = Hours;
                Hours++;
                bool success=(lasth != Hours);
               
                if (success)
                {
                    Minutes = 0;
                }
            }
        }

        private void btnMDownClick()
        {
            if (Minutes > 0)
            {
                Minutes--;
            }            
        }

        private void btnSUpClick()
        {            
            if (Seconds < 59)
            {
                Seconds++;
            }
            else
            {
                int lastm = Minutes;
                Minutes++;
                bool success = (lastm != Minutes);

                if (success)
                {
                    Seconds = 0;
                }
            }
        }

        private void btnSDownClick()
        {
            if (Seconds > 0)
            {
                Seconds--;
            }
        }

        private void btnMSUpClick()
        {
            if (MSeconds < 999)
            {
                MSeconds++;
            }
            else
            {
                int lasts = Seconds;
                Seconds++;
                bool success = (lasts != Seconds);

                if (success)
                {
                    MSeconds = 0;
                }
            }
        }

        private void btnMSDownClick()
        {
            if (MSeconds > 0)
            {
                MSeconds--;
            }
        }

        #endregion

        private void WaitNMSeconds(int mseconds)
        {
            if (mseconds < 1) return;
            DateTime _desired = DateTime.Now.AddMilliseconds(mseconds);
            while (DateTime.Now < _desired)
            {
                System.Windows.Forms.Application.DoEvents();
            }
        }

        private void txtH_TextChanged(object sender, EventArgs e)
        {
            try
            {
                DontFillTextBoxes = true;

                TextBox txt = (TextBox)sender;

                if (txt == txtH)
                {
                    int lasth = Hours;
                    int newh;

                    if (int.TryParse(txt.Text, out newh))
                    {
                        Hours = newh;
                    }

                    bool success = (lasth != Hours);

                    if (!success) { txt.Text = Hours.ToString("D2"); txt.SelectAll(); }


                }
                else if (txt == txtM)
                {
                    int lasth = Minutes;
                    int newh;

                    if (int.TryParse(txt.Text, out newh))
                    {
                        Minutes = newh;
                    }

                    bool success = (lasth != Minutes);

                    if (!success) { txt.Text = Minutes.ToString("D2"); txt.SelectAll(); }

                }
                else if (txt == txtS)
                {
                    int lasth = Seconds;
                    int newh;

                    if (int.TryParse(txt.Text, out newh))
                    {
                        Seconds = newh;
                    }

                    bool success = (lasth != Seconds);

                    if (!success) { txt.Text = Seconds.ToString("D2"); txt.SelectAll(); }

                }
                else if (txt == txtMS)
                {
                    int lasth = MSeconds;
                    int newh;

                    if (int.TryParse(txt.Text, out newh))
                    {
                        MSeconds = newh;
                    }

                    bool success = (lasth != MSeconds);

                    if (!success) { txt.Text = Hours.ToString("D3"); txt.SelectAll(); }
                }
            }
            finally
            {
                DontFillTextBoxes = false;
            }
        }              
    }
}
